__copyright__ = "Copyright (c) 2007 CollabNet, Inc. All rights reserved."

from app.scripts import BaseCreateRuntimeScript
import app.util
import service.util
import service.constants
import os
import commands
import sys

class CreateRuntimeScript(BaseCreateRuntimeScript):

    scriptDescription = "Setup Apache settings for Binary"

    def createRuntime(self):

        appName = service.constants.APPLICATION_BINARY
        binaryHostList = service.util.getApplicationHosts(appName)
        # Just return If Binary is not enabled with CTF
        if (len(binaryHostList) == 0):
           return

        confFile = self.cfg["HTTPD_CONF_FILE_PATH"]
        binaryListenPort = self.cfg["BINARY_LISTEN_PORT"]
        binaryHost = self.cfg["BINARY_HOST"]

        # Add a new line to the end of the Httpd Conf file to avoid line break present/not present confusion
        if (not self.checkHttpdConfEntry(confFile, '# IAF Integration')):
           self.appendHttpdConfFile(confFile, "")
        self.addHttpdConfEntry(confFile, '# IAF Integration')
        self.addHttpdConfEntry(confFile, 'RewriteRule ^/iaf$ /iaf/ [R]')
        self.addHttpdConfEntry(confFile, 'AllowEncodedSlashes On')
        self.addHttpdConfEntry(confFile, 'ProxyPass /iaf/ http://%s:%s/ nocanon'%(binaryHost, binaryListenPort))
#        self.addHttpdConfEntry(confFile, 'ProxyPass /stylesheets/ http://%s:%s/stylesheets/ nocanon'%(binaryHost, binaryListenPort))
#        self.addHttpdConfEntry(confFile, 'ProxyPass /assets/ http://%s:%s/assets/ nocanon'%(binaryHost, binaryListenPort))
        self.addHttpdConfEntry(confFile, 'ProxyPassReverse /iaf/ http://%s:%s/iaf/'%(binaryHost, binaryListenPort))

        if service.util.isSaaSEnabled():
            self.addHttpdModules()
            self.addHttpdService()
            self.restartHttpService()


    def addHttpdConfEntry(self, confFile, httpdConfEntry):

        if (self.checkHttpdConfEntry(confFile, httpdConfEntry)):
           return
        self.appendHttpdConfFile(confFile, httpdConfEntry)

    def appendHttpdConfFile(self, confFile, content):

        wfHandler = open (confFile, 'a')
        wfHandler.write(content + "\n")
        wfHandler.close()
        return

    def checkHttpdConfEntry(self, confFile, httpdConfEntry):

        fHandler = open(confFile, 'r')
        httpdConfLines = fHandler.read().split("\n")
        fHandler.close()

        if (len(filter(lambda x: x == httpdConfEntry, httpdConfLines)) > 0):
           return True
        return False

    def addHttpdModules(self):
        if os.path.exists('/etc/SuSE-release'):
           # Suse doesnot have rewrite/proxy modules by default. So add them using a2enmod command.
           for httpdModule in ('rewrite', 'proxy', 'ssl', 'proxy_filter', 'proxy_connect', 
                               'proxy_http', 'proxy_balancer', 'proxy_ajp', 'proxy_ftp'):
               runCommand = "/usr/sbin/a2enmod %s"%(httpdModule)
               (retVal, outputVal) = commands.getstatusoutput(runCommand)
               if retVal != 0:
                  self.logMesg("error", "Error occurred while enabling apache module [%s]!!"%(httpdModule))
                  self.logMesg("error", "Reason: %s"%outputVal) 
                  sys.exit(1)

           # To enable SSL flag in apache
           runCommand = "/usr/sbin/sysconf_addword /etc/sysconfig/apache2 APACHE_SERVER_FLAGS SSL"
           (retVal, outputVal) = commands.getstatusoutput(runCommand)
           if retVal != 0:
              self.logMesg("error", "Error occurred while enabling SSL flag in apache!!")
              self.logMesg("error", "Reason: %s"%outputVal) 
              sys.exit(1)

        return

    def addHttpdService(self):

        serviceScript = self.cfg['HTTPD_SERVICE_CMD']
        if "redhat-7" in self.cfg["PLATFORM"]:
            runCommand  = "%s enable" %serviceScript
        else:
            if not os.path.isfile(serviceScript):
                return

            serviceName = os.path.basename(serviceScript)
            runCommand  = "/sbin/chkconfig --add %s"%(serviceName)
        commands.getstatusoutput(runCommand)


    def restartHttpService(self):

        serviceScript = self.cfg['HTTPD_SERVICE_CMD']
        runCommand    = "%s reload"%(serviceScript)
        (retVal, outputVal) = commands.getstatusoutput(runCommand)

        if retVal != 0:
           self.logMesg("error", "Unable to start apache service. Exited with error code %s %s!!"%(retVal,runCommand))
           self.logMesg("error", "Reason: %s"%outputVal) 
           sys.exit(1)
