__copyright__ = "Copyright (c) 2007 CollabNet, Inc. All rights reserved."

from app.scripts import BaseCreateRuntimeScript
import app.util
import service.util
import service.constants
import os
import commands
import re

class CreateRuntimeScript(BaseCreateRuntimeScript):

    scriptDescription = "Setup PostgreSQL pg_hba.conf for Binary"

    def createRuntime(self):

        appName = service.constants.APPLICATION_BINARY
        if not(service.util.isApplicationEnabledLocally(appName)
                and (service.util.isSaaSEnabled() or self.cfg['IS_SCM_SERVER'] == "true")):
           return

        dbUser = self.cfg['IAF_DBUSER']
        dbName = self.cfg['IAF_DBNAME']
        hostIP = service.util.getServiceConfiguration().getHostIP(self.cfg['DATABASE_HOST'])

        confFile = os.path.join(self.cfg['PGSQL_DATA_DIR'], 'pg_hba.conf')
        hbaEntry = "host    %s    %s    %s 255.255.255.255   md5" %(dbName, dbUser,hostIP)
        # Adds "host binarydb binaryusr samehost md5" if not present
        if (self.checkHbaEntry(confFile, hbaEntry) == False):
           self.addHbaEntry(confFile, hbaEntry)
        hbaEntry = "host    %s    %s    127.0.0.1 255.255.255.255   md5" %(dbName, dbUser)

        # Adds "host binarydb binaryusr samehost md5" if not present
        if (self.checkHbaEntry(confFile, hbaEntry) == False):
           self.addHbaEntry(confFile, hbaEntry)

        # Removes the conflicting entry "host all all 127.0.0.1/32 ident" if present.
        self.removeHbaConflict(confFile)

    def addHbaEntry(self, confFile, hbaEntry):

        aHandler = open(confFile, "a")
        aHandler.write("\n"  + "# Entry for IAF  Database" + "\n")
        aHandler.write(hbaEntry + "\n")
        aHandler.close()

    def removeHbaConflict(self, confFile):

        fHandler = open(confFile, "r")
        hbaFileLines = fHandler.read().split("\n")
        fHandler.close()

        conflictRegExp  = "^host\s+all\s+all\s+127.0.0.1\/32\s+ident"
        conflictMatcher = re.compile(conflictRegExp)
        if (len(filter(conflictMatcher.search, hbaFileLines)) > 0):
           runCommand = 'sed -i -r "s/%s/# &/" %s'%(conflictRegExp, confFile)
           (retVal, outputVal) = commands.getstatusoutput(runCommand)

    def checkHbaEntry(self, confFile, hbaEntry):

        fHandler = open(confFile, "r")
        hbaFileLines = fHandler.read().split("\n")
        fHandler.close()

        # Match the Entry as it begins with the line
        hbaEntry = '^' + hbaEntry
        matcher = re.compile(hbaEntry)
        if (len(filter(matcher.search, hbaFileLines)) > 0):
           return True
        return False

