__copyright__ = "Copyright (c) 2007 CollabNet, Inc. All rights reserved."

from app.scripts import BaseCreateRuntimeScript
import app.util
import service.util
import service.constants
import os
import commands
import sys
import time

class CreateRuntimeScript(BaseCreateRuntimeScript):

    scriptDescription = "Setup PostgreSQL for Binary"

    def createRuntime(self):

        appName = service.constants.APPLICATION_BINARY

        self.addPgsqlService()

        self.startPgsqlService()

        dbUser = self.cfg['IAF_DBUSER']
        dbName = self.cfg['IAF_DBNAME']
        # Check if "binary" user is created in Postgres. If not create.
        isDbUserExists = self.checkDbUserExists(dbUser)

        if isDbUserExists == False:
           self.createDbUser(dbUser)
        #self.createDbPasswd()

        # Check if "reviewdb" tablespace/db is created in Postgres. If not create.
        isDbExists = self.checkDbExists(dbName)

        if isDbExists == False:
           self.createDatabase(dbUser, dbName)

        return

    def createDatabase(self, dbUser, dbName):
        runCommand = 'su - postgres -c "createdb --encoding=UTF8 --owner=%s %s"'%(dbUser, dbName)
        (retVal, outputVal) = commands.getstatusoutput(runCommand)
        if (retVal != 0):
           self.logMesg("error", "Error occurred while creating postgresql database [%s]. Exited with error code %s!!"%(dbName, retVal))
           self.logMesg("error", "Reason: %s"%outputVal) 
           sys.exit(1)
        else:
           self.logMesg("debug", "Postgresql Database [%s] created successfully."%dbName)


    def checkDbExists(self, dbName):
        runCommand = 'su - postgres -c "psql postgres -tAc \\"select \'db_object_found\' from pg_database where datname=\'%s\'\\" | grep --count \'db_object_found\'"'%dbName
        (retVal, outputVal) = commands.getstatusoutput(runCommand)
        try:
           return True if int(outputVal) > 0 else False
        except:
           return False

    def createDbUser(self, dbUser):
        runCommand = 'su - postgres -c "createuser --no-superuser --createdb --no-createrole %s"'%dbUser
        (retVal, outputVal) = commands.getstatusoutput(runCommand)
        if (retVal != 0):
           self.logMesg("error", "Error occurred while creating postgresql user [%s]. Exited with error code %s!!"%(dbUser, retVal))
           self.logMesg("error", "Reason: %s"%outputVal) 
           sys.exit(1)
        else:
           self.logMesg("debug", "Postgresql User [%s] created successfully."%dbUser)

    def createDbPasswd(self):
        # Default Postgresql Port
        pgPort = '5432';
        if (self.cfg['DATABASE_TYPE'] == 'pgsql'):
          pgPort = self.cfg['DATABASE_PORT']
        runCommand = """echo "ALTER ROLE \\"%s\\" WITH PASSWORD '%s'" |psql -p %s -U postgres --tuples-only | grep '^ALTER ROLE$'""" % \
                (self.cfg['IAF_DBUSER'], self.cfg['IAF_DBPASS'],
                pgPort)

        (retVal, outputVal) = commands.getstatusoutput(runCommand)
        if (retVal != 0):
           self.logMesg("error", "Error occurred while creating postgresql user [%s]'s password. Exited with error code %s!!"%(self.cfg['IAF_DBUSER'], retVal))
           self.logMesg("error", "Reason: %s"%outputVal) 
           sys.exit(1)
        else:
           self.logMesg("debug", "Postgresql User [%s]'s password created successfully."%self.cfg['IAF_DBUSER'])

    def checkDbUserExists(self, dbUser):
        runCommand = 'su - postgres -c "psql postgres -tAc \\"select \'db_object_found\' from pg_roles where rolname=\'%s\'\\" | grep --count \'db_object_found\'"'%dbUser
        (retVal, outputVal) = commands.getstatusoutput(runCommand)
        try:
           return True if int(outputVal) > 0 else False
        except:
           return False

    def addPgsqlService(self):

        serviceScript = self.cfg['PGSQL_SERVICE_CMD'].split(" ")[0]

        if "redhat-7" in self.cfg["PLATFORM"]:
            tfSystemd = os.path.join(self.cfg["RUNTIME_SCRIPTS"], 'tfsystemd')
            if (service.util.isServiceEnabledLocally(service.constants.SERVICE_PGSQL)):
                serviceName = "ctf-postgresql-%s" %self.cfg["PGSQL_VERSION"]
            else:
                serviceName = "postgresql-%s" %self.cfg["PGSQL_VERSION"]
            pgInitAddCmd = "%s %s enable" %(tfSystemd, serviceName)
        else:
            if not os.path.isfile(serviceScript):
               return
            serviceName = os.path.basename(serviceScript)
            pgInitAddCmd = "/sbin/chkconfig --add %s"%(serviceName) 

        commands.getstatusoutput(pgInitAddCmd)

    def startPgsqlService(self):

        serviceScript = self.cfg['PGSQL_SERVICE_CMD'].split(" ")[0]
        if "redhat-7" in self.cfg["PLATFORM"]:
            tfSystemd = os.path.join(self.cfg["RUNTIME_SCRIPTS"], 'tfsystemd')
            if (service.util.isServiceEnabledLocally(service.constants.SERVICE_PGSQL)):
                serviceName = "ctf-postgresql-%s" %self.cfg["PGSQL_VERSION"]
            else:
                serviceName = "postgresql-%s" %self.cfg["PGSQL_VERSION"] 
            runCommand = "%s %s restart" %(tfSystemd, serviceName)
        else:
            runCommand    = "%s restart"%(serviceScript)
        self.runOSCommand(runCommand)
        # Default Postgresql Port, if case on gerrit is installed.
        pgPort = '5432';
        if (self.cfg['DATABASE_TYPE'] == 'pgsql'):
          pgPort = self.cfg['DATABASE_PORT']
        service.util.waitForServerUp('localhost', pgPort, 30)

    def runOSCommand(self, cmd):
        pipe = os.popen('{ ' + cmd + '; } 2>&1', 'r')

        try:
           output = pipe.readline()
        finally:
           status = pipe.close()

        if status is None: status = 0
        if output[-1:] == '\n': output = output[:-1]
        return status, output 
